import { OwnedAudioFrameBuffer, AudioFrameBufferInfo } from './proto/audio_frame_pb.js';
import '@bufbuild/protobuf';
import './proto/track_pb.js';
import './proto/stats_pb.js';
import './proto/e2ee_pb.js';
import './proto/handle_pb.js';

declare class AudioFrame {
    data: Int16Array;
    sampleRate: number;
    channels: number;
    samplesPerChannel: number;
    constructor(data: Int16Array, sampleRate: number, channels: number, samplesPerChannel: number);
    static create(sampleRate: number, channels: number, samplesPerChannel: number): AudioFrame;
    /** @internal */
    static fromOwnedInfo(owned: OwnedAudioFrameBuffer): AudioFrame;
    /** @internal */
    protoInfo(): AudioFrameBufferInfo;
}
/**
 * Combines one or more `rtc.AudioFrame` objects into a single `rtc.AudioFrame`.
 *
 * This function concatenates the audio data from multiple frames, ensuring that all frames have
 * the same sample rate and number of channels. It efficiently merges the data by preallocating the
 * necessary memory and copying the frame data without unnecessary reallocations.
 *
 * @param buffer - a single AudioFrame or list thereof
 */
declare const combineAudioFrames: (buffer: AudioFrame | AudioFrame[]) => AudioFrame;

export { AudioFrame, combineAudioFrames };
