import { FfiClient, FfiHandle } from "./ffi_client.js";
import { AudioFrameBufferInfo } from "./proto/audio_frame_pb.js";
class AudioFrame {
  // note: if converting from Uint8Array to Int16Array, *do not* use buffer.slice!
  // it is marked unstable by Node and can cause undefined behaviour, such as massive chunks of
  // noise being added to the end.
  // it is recommended to use buffer.subarray instead.
  // XXX(nbsp): add this when writing proper docs
  constructor(data, sampleRate, channels, samplesPerChannel) {
    this.data = data;
    this.sampleRate = sampleRate;
    this.channels = channels;
    this.samplesPerChannel = samplesPerChannel;
  }
  static create(sampleRate, channels, samplesPerChannel) {
    const data = new Int16Array(channels * samplesPerChannel);
    return new AudioFrame(data, sampleRate, channels, samplesPerChannel);
  }
  /** @internal */
  static fromOwnedInfo(owned) {
    const info = owned.info;
    const len = info.numChannels * info.samplesPerChannel * 2;
    const data = FfiClient.instance.copyBuffer(info.dataPtr, len);
    new FfiHandle(owned.handle.id).dispose();
    return new AudioFrame(
      new Int16Array(data.buffer),
      info.sampleRate,
      info.numChannels,
      info.samplesPerChannel
    );
  }
  /** @internal */
  protoInfo() {
    return new AudioFrameBufferInfo({
      dataPtr: FfiClient.instance.retrievePtr(new Uint8Array(this.data.buffer)),
      sampleRate: this.sampleRate,
      numChannels: this.channels,
      samplesPerChannel: this.samplesPerChannel
    });
  }
}
const combineAudioFrames = (buffer) => {
  if (!Array.isArray(buffer)) {
    return buffer;
  }
  buffer = buffer;
  if (buffer.length === 0) {
    throw new Error("buffer is empty");
  }
  const sampleRate = buffer[0].sampleRate;
  const channels = buffer[0].channels;
  let totalSamplesPerChannel = 0;
  for (const frame of buffer) {
    if (frame.sampleRate != sampleRate) {
      throw new Error(`sample rate mismatch: expected ${sampleRate}, got ${frame.sampleRate}`);
    }
    if (frame.channels != channels) {
      throw new Error(`channel mismatch: expected ${channels}, got ${frame.channels}`);
    }
    totalSamplesPerChannel += frame.samplesPerChannel;
  }
  const data = new Int16Array(buffer.map((x) => [...x.data]).flat());
  return new AudioFrame(data, sampleRate, channels, totalSamplesPerChannel);
};
export {
  AudioFrame,
  combineAudioFrames
};
//# sourceMappingURL=audio_frame.js.map