import { PartialMessage } from '@bufbuild/protobuf';
import { TypedEventEmitter } from '@livekit/typed-emitter';
export { FfiHandle, livekitDispose as dispose } from './napi/native.d.js';
import { FfiEvent, FfiRequest } from './proto/ffi_pb.js';
export { FfiResponse } from './proto/ffi_pb.js';
import './proto/room_pb.js';
import './proto/participant_pb.js';
import './proto/handle_pb.js';
import './proto/track_pb.js';
import './proto/stats_pb.js';
import './proto/e2ee_pb.js';
import './proto/video_frame_pb.js';
import './proto/audio_frame_pb.js';
import './proto/rpc_pb.js';
import './proto/track_publication_pb.js';

type FfiClientCallbacks = {
    ffi_event: (event: FfiEvent) => void;
};
declare enum FfiClientEvent {
    FfiEvent = "ffi_event"
}
declare global {
    var _ffiClientInstance: FfiClient | undefined;
}
declare const FfiClient_base: new () => TypedEventEmitter<FfiClientCallbacks>;
declare class FfiClient extends FfiClient_base {
    /** @internal */
    static get instance(): FfiClient;
    constructor();
    request<T>(req: PartialMessage<FfiRequest>): T;
    copyBuffer(ptr: bigint, len: number): Uint8Array;
    retrievePtr(data: Uint8Array): bigint;
    waitFor<T>(predicate: (ev: FfiEvent) => boolean): Promise<T>;
}

export { FfiClient, type FfiClientCallbacks, FfiClientEvent, FfiEvent, FfiRequest };
